﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MouseEvent2
{
    public partial class Form1 : Form
    {
        private Bitmap offScreenBmp;
        private Graphics offScreenDC;

        private Font myFont = new System.Drawing.Font("Lucida Console", 12, FontStyle.Regular);
        private Brush myBrush = new SolidBrush(Color.White);
        private SolidBrush backBrush;

        private int zoomFactor = 0;
        private int xFactor = 0;
        private int yFactor = 0;

        private const int canvasWidth = 8192;
        private const int canvasHeight = 8192;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Set up graphic objects
            offScreenBmp = new Bitmap(canvasWidth, canvasHeight); //panel1.Width, panel1.Height);
            offScreenDC = Graphics.FromImage(offScreenBmp);
            backBrush = new SolidBrush(Color.Black);

            // Paint initial map
            drawMap();

            // MouseWheel event handler
            this.panel1.MouseWheel += new System.Windows.Forms.MouseEventHandler(this.panel1_MouseWheel);
        }

        private void panel1_MouseDown(object sender, MouseEventArgs e)
        {
            panel1.Focus();
        }

        private void panel1_MouseWheel(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            // Update the drawing based upon the mouse wheel scrolling.

            System.Drawing.Drawing2D.Matrix translateMatrix = new System.Drawing.Drawing2D.Matrix();
            //PointF rotatePoint = new PointF(0.0f, 0.0f);

            if ((e.Delta > 0) && (zoomFactor < 9))
            {
                //translateMatrix.Translate(0, numberOfPixelsToMove);
                float scaleX = 1.25F;
                float scaleY = 1.25F;
                translateMatrix.Scale(scaleX, scaleY);
                //translateMatrix.RotateAt(15, rotatePoint);
                offScreenDC.MultiplyTransform(translateMatrix);
                zoomFactor++;
            }
            else if ((e.Delta < 0) && (zoomFactor > -9))
            {
                float scaleX = 0.8F;
                float scaleY = 0.8F;
                translateMatrix.Scale(scaleX, scaleY);
                //translateMatrix.RotateAt(-15, rotatePoint);
                offScreenDC.MultiplyTransform(translateMatrix);
                zoomFactor--;
            }

            wZoomFactor.Text = zoomFactor.ToString();

            //panel1.Invalidate();
            drawMap();
        }

        private void panel1_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            System.Drawing.Drawing2D.Matrix translateMatrix = new System.Drawing.Drawing2D.Matrix();

            int step = 16;

            switch (e.KeyCode)
            {
                case Keys.W:
                    label1.Text = "You pressed W";
                    if (yFactor > 0) {
                        translateMatrix.Translate(0, step);
                        yFactor--;
                    }
                    break;
                case Keys.A:
                    label1.Text = "You pressed A";
                    if (xFactor > 0) {
                        translateMatrix.Translate(step, 0);
                        xFactor--;
                    }
                    break;
                case Keys.S:
                    label1.Text = "You pressed S";
                    if (yFactor < 256) {
                        translateMatrix.Translate(0, -step);
                        yFactor++;
                    }
                    break;
                case Keys.D:
                    label1.Text = "You pressed D";
                    if (xFactor < 256) {
                        translateMatrix.Translate(-step, 0);
                        xFactor++;
                    }
                    break;
            }

            wXScroll.Text = xFactor.ToString();
            wYScroll.Text = yFactor.ToString();

            offScreenDC.MultiplyTransform(translateMatrix);
            //panel1.Invalidate();
            drawMap();
        }

        private void drawMap()
        {
            panel1_Paint(null, null);
        }

        private void panel1_Paint(object sender, PaintEventArgs e)
        {
            Graphics clientDC = panel1.CreateGraphics();

            offScreenDC.FillRectangle(backBrush, 0, 0, canvasWidth, canvasHeight);

            // Graphics Painting
            for (int x = 0; x < 192; x += 16) {
                for (int y = 0; y < 192; y += 16) {
                    if ((x != 32) || (y != 32)) {
                        offScreenDC.DrawString("#", myFont, myBrush, x, y);
                    }
                }
            }
            offScreenDC.DrawString("@", myFont, myBrush, 32, 32);

            // Let's try a long tunnel
            for (int x = 192; x < 1024; x += 16) {
                offScreenDC.DrawString("#", myFont, myBrush, x, 64);
                offScreenDC.DrawString(".", myFont, myBrush, x, 80);
                offScreenDC.DrawString("#", myFont, myBrush, x, 96);
            }

            // Paint the four corners
            offScreenDC.DrawString("B", myFont, myBrush, 0, 4080);
            offScreenDC.DrawString("C", myFont, myBrush, 4080, 0);
            offScreenDC.DrawString("D", myFont, myBrush, 4080, 4080);

            // graphics test
            Image newImage = Image.FromFile("test.bmp");
            for (int x = 256; x < 1024; x += 64) {
                offScreenDC.DrawImage(newImage, new Point(x, 256));
            }

            /*
            // four corners
            offScreenDC.DrawString("#", myFont, myBrush, -4096, -4096);
            offScreenDC.DrawString("#", myFont, myBrush, -4096,  4096);
            offScreenDC.DrawString("#", myFont, myBrush,  4096, -4096);
            offScreenDC.DrawString("#", myFont, myBrush,  4096,  4096);

            for (int x = -4096; x < 4096; x += 512)
            {
                for (int y = -4096; y < 4096; y += 512)
                {
                    offScreenDC.DrawString(".", myFont, myBrush, x, y);
                }
            }
            */

            // now move from back buffer to front
            clientDC.DrawImage(offScreenBmp, 0, 0);
        }

        private void Form1_ResizeEnd(object sender, EventArgs e)
        {
            offScreenBmp = new Bitmap(panel1.Width, panel1.Height);
            offScreenDC = Graphics.FromImage(offScreenBmp);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            System.Drawing.Drawing2D.Matrix translateMatrix = new System.Drawing.Drawing2D.Matrix();
            PointF rotatePoint = new PointF(256f, 256f);
            translateMatrix.RotateAt(0.5f, rotatePoint);
            offScreenDC.MultiplyTransform(translateMatrix);
            drawMap();
        }

        private void Form1_Resize(object sender, EventArgs e)
        {
            Form1_ResizeEnd(sender, e);
        }

        private void labelZoomFactor_Click(object sender, EventArgs e)
        {

        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            timer1.Enabled = false;
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            timer1.Enabled = true;
        }

    }
}
